import Attrs, { AttrsObject } from './Attrs';
import Override from './Override';
export declare enum CtrlType {
    TITLE = 0,
    ETC = 1,
}
export interface LinkedListNode<T> {
    prev: LinkedListNode<T>;
    next: LinkedListNode<T>;
}
export declare function validateTitle(tagName: string, value: string): void;
export declare function validateEtc(tagName: string, value: AttrsObject): void;
export declare const validator: {
    [x: number]: ((tagName: string, value: string) => void) | ((tagName: string, value: AttrsObject) => void);
};
export interface Listener {
    (): void;
}
export interface AddListener {
    (listener: Listener): any;
}
export interface RemoveListener {
    (listener: Listener, _addListenerRet?: any): any;
}
export interface ControllerOptions {
    addListener?: AddListener;
    removeListener?: RemoveListener;
    close?: boolean;
}
export interface RenderFunc<T> {
    (): T;
}
declare class Controller<T> implements LinkedListNode<Controller<T>> {
    id: number;
    override: Override;
    selector: string;
    prev: Controller<T>;
    next: Controller<T>;
    type: CtrlType;
    attrs: Attrs;
    tagName: string;
    needsToClose: boolean;
    render: RenderFunc<T>;
    addListener: AddListener;
    removeListener: RemoveListener;
    /**
     * Controller constructor.
     *
     * Controller is an object that manages an element in `<head>` element.
     * If multiple controllers are bound to one element, only the last bound
     * controller is used.
     * The last bound controller is called top controller.
     *
     * When top controller is removed, previous controller of that one becomes
     * the new top controller.
     *
     * @constructs Controller
     * @param {CtrlType} type Type of controller.
     * @param {string} tagName Element tag name.
     * @param {object} attrs Object containing key-value attribute pairs, which is
     * used to find element to bind controller to.
     * @param {function} render A function that returns new element value.
     * The type of return value depends on controller type.
     * @param {object} options Controller options.
     * @param {addListener} options.addListener A function that adds controller
     * listener to event emitter.
     * @param {removeListener} options.removeListener A function that removes
     * controller listener.
     * @return {Controller} a new Controller instance.
     */
    constructor(type: CtrlType, tagName: string, attrs: AttrsObject, render: RenderFunc<T>, options?: ControllerOptions);
    insertAfter(node: Controller<T>): void;
    unlink(): void;
}
export default Controller;
