import { AttrsObject } from './Attrs';
import Controller from './Controller';
import Override from './Override';
export declare class OverrideListener {
    private _controllerListeners;
    private _override;
    private _listener;
    private _listening;
    private _addListenerRet;
    constructor(override: Override);
    onUpdate(): void;
    addControllerListener(controllerListener: ControllerListener): void;
    removeControllerListener(controllerListener: ControllerListener): void;
    isReadyToRemove(): boolean;
}
export interface ControllerListenerOptions {
    controller: Controller<any>;
    overrideListener: OverrideListener;
    handler: EventHandler<any>;
}
export declare class ControllerListener {
    private _overrideListener;
    private _controller;
    private _listening;
    private _onUpdate;
    private _onStop;
    private _listener;
    private _addListenerRet;
    constructor({controller, overrideListener, handler}: ControllerListenerOptions);
    onUpdate(): this;
    onStop(): this;
    startListening(): this;
    stopListening(): this;
    isListening(): boolean;
    isReadyToRemove(): boolean;
}
export interface EventHandler<T> {
    onUpdate(controller: Controller<T>): void;
    onStop?(controller: Controller<T>): void;
}
export declare type TitleEventHandler = EventHandler<string>;
export declare type EtcEventHandler = EventHandler<AttrsObject>;
export interface EventHandlers {
    [index: number]: EventHandler<any>;
}
declare class EventManager {
    private _overrideListeners;
    private _controllerListeners;
    private _eventHandlers;
    constructor(titleHandler: TitleEventHandler, etcHandler: EtcEventHandler);
    isControllerListening(controller: Controller<any>): boolean;
    startListeningController(controller: Controller<any>): void;
    stopListeningController(controller: Controller<any>): void;
    removeController(controller: Controller<any>): void;
    updateOverride(override: Override): void;
    removeOverride(override: Override): void;
    destroy(): void;
    private _ensureOverrideListener(override);
    private _getControllerListener(controller);
    private _ensureControllerListener(controller);
}
export default EventManager;
