import { AttrsObject } from './Attrs';
import Controller from './Controller';
import Override, { OverrideOptions } from './Override';
import { TitleEventHandler, EtcEventHandler } from './EventManager';
export interface Observer {
    onAddController(controller: Controller<any>): void;
    onUpdate(override: Override): void;
    onPreRemoveOverride(override: Override): void;
}
/**
 * @callback addListener
 * @param {function} listener A function to be bound to event emitter.
 * @return {*} A value to pass to {@link removeListener}.
 */
/**
 * @callback removeListener
 * @param {function} listener A function to be removed from event emitter.
 * @param {*} addListenerReturnValue Return value of addListener
 */
export default class Hairdresser {
    /**
     * Create a Hairdresser instance.
     *
     * @return {Hairdresser} Hairdresser instance.
     */
    static create(): Hairdresser;
    _topControllers: {
        [index: string]: Controller<any>;
    };
    _observerList: Observer[];
    _generateOverrideId: () => number;
    _generateControllerId: () => number;
    /**
     * Hairdresser constructor.
     *
     * @constructs Hairdresser
     * @return {Hairdresser} a new Hairdresser instance.
     */
    constructor();
    /**
     * Create a override instance for overriding current top controllers.
     *
     * When override listener receives an event, top controllers in override
     * rerenders their elements.
     *
     * @param {object} options Override options
     * @param {addListener} options.addListener A function that adds override
     * listener to event listener.
     * @param {removeListener} options.removeListener A function that removes
     * override listener.
     * @return {Override} A override instance.
     */
    override(options?: OverrideOptions): Override;
    /**
     * Renders HTML elements as a markup string.
     *
     * @return {string} the HTML markup
     */
    renderToString(): string;
    /**
     * Renders HTML elements into the DOM and watch further changes
     * of the hairdresser instance.
     *
     * @param {string | HTMLElement} selectorOrElement HTMLElement instance or
     * a selector to find an HTMLElement to append rendered elements to.
     * Defaults to the `<head>` element.
     * @return {function} Restore changes and stop watching the hairdresser
     * instance function.
     */
    render(selectorOrElement?: string | HTMLElement): () => void;
    _addControllerToOverride<T>(override: Override, controller: Controller<T>): void;
    _getActiveController(tagName: string, attrs?: AttrsObject): Controller<any>;
    _renderAndListen(titleHandler: TitleEventHandler, etcHandler?: EtcEventHandler): () => void;
    private _getActiveControllers();
    private _addObserver(observer);
    private _renderOnce(titleHandler, etcHandler);
}
